"""
Created on: 25-6-2021 12:45

@author: IvS
"""
import itertools

from pydsol.basic_logger import get_module_logger

logger = get_module_logger(__name__)


class AbstractSimEvent(object):
    """ The base class for a SimEvent. The constructor of the event stores the time the event must be executed and
    the object and method to invoke."""
    # Class parameters
    id_iter = itertools.count()

    min_priority = 1
    normal_priority = 5
    max_priority = 10

    def __init__(self, absolute_execution_time, priority=normal_priority):
        """

        Parameters
        ----------
        absolute_execution_time: int
            the time the event has to be executed.
        priority: int
            the priority of the event. Default is 5.
            Min priority is 1, normal priority is 5, and max priority is 10.

        """
        self.absolute_execution_time = absolute_execution_time

        if self.min_priority <= priority <= self.max_priority:
            self.priority = priority
        else:
            raise ValueError("Priority must be between {0} and {1}.".format(self.min_priority, self.max_priority))

        self.id = next(self.id_iter)
