"""
Created on: 25-6-2021 11:16

@author: IvS
"""
from pytreemap import TreeSet
from pydsol.basic_logger import get_module_logger

logger = get_module_logger(__name__)


class EventList(TreeSet):
    """ This class defines the event list of the simulation model. This class inherits the attributes
    and methods from TreeSet. Comparator is also defined in this class."""

    def __init__(self):
        super().__init__(comparator=self._cmp)

    def remove_first(self):
        """Removes and return the first (lowest) element currently in this sorted set."""
        first = self.first()
        self.remove(first)
        logger.debug("First key is removed")
        return first

    def remove_last(self):
        """Removes and return the last (highest) element currently in this sorted set."""
        last = self.last()
        self.remove(last)
        logger.debug("Last key is removed")
        return last

    # Override
    def first(self):
        """Returns the first (lowest) element currently in this sorted set."""
        try:
            return super().first()
        except KeyError:
            return None

    # Override
    def last(self):
        """Returns the last (highest) element currently in this sorted set."""
        try:
            return super().last()
        except KeyError:
            return None

    # Create the comparator
    @staticmethod
    def _cmp(this, other):
        """
        This function defines a compare method by which the sorted list can compare priority of an event.

        Parameters
        ----------
        this: object
            this event
        other: object
            other event

        """
        if this == other:
            return 0
        elif this.absolute_execution_time < other.absolute_execution_time:
            return -1
        elif this.absolute_execution_time > other.absolute_execution_time:
            return 1
        elif this.priority < other.priority:
            return 1
        elif this.priority > other.priority:
            return -1
        elif this.id < other.id:
            return -1
        elif this.id > other.id:
            return 1
        else:
            raise AttributeError("This cannot happen!")
