"""
Created on: 25-6-2021 11:33

@author: IvS
"""
from pydsol.basic_logger import get_module_logger
from pydsol.pydsol_core.AbstractSimEvent import AbstractSimEvent

logger = get_module_logger(__name__)



class SimEvent(AbstractSimEvent):
    """The SimEvent forms the essential scheduling mechanism for pydsol. Objects do not invoke methods directly on each
     other; they bundle the object on which the method is planned to be invoked together with the arguments and the name
    of the method in a SimEvent. The SimEvent is then stored in the eventlist and executed."""

    def __init__(self, execution_time, source, target, method, priority=None, **kwargs):
        """

        Parameters
        ----------
        execution_time: int
            the absolute time the event has to be executed.
        source: object
            source that creates the method.
        target:  object, or class for __init__
            target the target on which a state change is scheduled.
        method: str
            method is the method which embodies the state changes.
        priority: int
            priority of the event, default is normal priority.
        kwargs:
            kwargs are the keyword arguments that are used to invoke the method with."""
        if priority is None:
            super().__init__(execution_time)
        else:
            super().__init__(execution_time, priority)

        self.source = source
        self.target = target
        self.method_name = method
        self.kwargs = kwargs

    def execute(self):
        """Execute a SimEvent for either __init__ for creating new objects of a class or for executing a method."""
        logger.debug("Time {0:.2f}: Event {1} with target {2} and method {3} is executed".format(
            self.absolute_execution_time, self.id, self.target, self.method_name))

        # try:
        if self.method_name == "__init__":
            # if not hasattr(self.target, "__name__"):
            if not isinstance(self.target, type):
                raise AttributeError(f"{self.target} is an instance, should be a Class")
            self.target(**self.kwargs)
        else:
            try:
                method = getattr(self.source, self.method_name)
            except AttributeError:
                raise AttributeError(f"{self.method_name} is not accessible for {self.source}")
            else:
                method(self.target, **self.kwargs)
        # except Exception:
        #     raise Exception("This does not work")
