"""
Created on: 22-7-2021 10:22

@author: IvS
"""
import itertools

from pydsol.basic_logger import get_module_logger
import logging

logger = get_module_logger(__name__)


class Entity(object):
    """This class defines a basic entity for a discrete event simulation model."""
    id_iter = itertools.count(1)

    def __init__(self, simulator, speed=1.4, **kwargs):
        """

        Parameters
        ----------
        simulator: Simulator object
            simulator of the model.
        speed:
            speed of the entity. Default is 1.4 m/s.
        kwargs:
             kwargs are the keyword arguments that are used to expand the entity class.
             * source_entity: object
                source of which the entity should exit after creation. If keyword is not existing, the entity has no
                source to exit from.
        """
        self.simulator = simulator
        self.id = next(self.id_iter)
        self.name = "{0} {1}".format(self.__class__.__name__, str(self.id))
        self.speed = speed
        self.kwargs = kwargs

        if "source_entity" in kwargs:
            logger.info("Time {0:.2f}: {1} is created at {2}".format(self.simulator.simulation_time, self.name,
                                                                     kwargs["source_entity"].name))
            self.next_event_now(kwargs["source_entity"], "exit_source")

    def next_event_now(self, source, method, **kwargs):
        """Schedules next event of the entity now."""
        next_event = self.simulator.create_simevent_now(source, self, method, **kwargs)
        self.simulator.schedule_event(next_event)
