"""
Created on: 12-8-2021 10:59

@author: IvS
"""
import itertools
import numpy as np

from pydsol.pydsol_model.resource import Resource
from pydsol.pydsol_model.queue import Queue
from pydsol.pydsol_model.node import Node

import logging
from pydsol.basic_logger import get_module_logger

logger = get_module_logger(__name__, level=logging.DEBUG)


class ServerWithResourceObject(object):
    id_iter = itertools.count(1)

    def __init__(self, simulator, capacity=1, distribution=np.random.triangular, processing_time=(0.1, 0.2, 0.3), transfer_in_time=0, **kwargs):
        self.simulator = simulator

        self.id = next(self.id_iter)
        self.name = "{0} {1}".format(self.__class__.__name__, str(self.id))
        if "name" in kwargs:
            self.name = kwargs["name"]

        self.processing_time = processing_time
        self.distribution = distribution

        self.capacity = capacity
        self.transfer_in_time = transfer_in_time
        self.kwargs = kwargs

        # Transport
        self.input_links = []
        self.output_links = []

        self.input_node = Node(self.simulator)
        self.output_node = Node(self.simulator)

        # Queue
        self.input_queue = Queue(self)
        self.resources = []
        for _ in range(self.capacity):
            #TODO make distribution and processing time for multiple resources (in list)
            resource = Resource(self.simulator, self.input_queue, self.distribution, self.processing_time, self.transfer_in_time, **kwargs)
            resource.input_links = self.input_links
            resource.output_links = self.output_links

            self.resources.append(resource)

    def enter_input_node(self, entity):
        event = self.simulator.create_simevent_now(self, entity, "seize_resource")
        self.simulator.schedule_event(event)

    def seize_resource(self, entity):
        not_seized = [resource for resource in self.resources if not resource.resource_seized]

        if len(not_seized) == 0:
            self.input_queue.contents.append(entity)
            logger.debug(
                "Time {0:.2f}: {1} added to the queue".format(self.simulator.simulation_time, entity.name))
        else:
            resource_to_seize = not_seized[0]
            if len(self.input_queue.contents) == 0:
                resource_to_seize.resource_seized = True
                event = self.simulator.create_simevent_now(resource_to_seize, entity, "enter_resource")
                self.simulator.schedule_event(event)
                logger.debug(
                    "Time {0:.2f}: {1} seized {2}".format(self.simulator.simulation_time, entity.name,
                                                          resource_to_seize.name))
            else:
                resource_to_seize.resource_seized = True
                event = self.simulator.create_simevent_now(resource_to_seize, self.input_queue.contents[0],
                                                           "enter_resource")
                self.simulator.schedule_event(event)
                logger.debug(
                    "Time {0:.2f}: {1} seized {2}".format(self.simulator.simulation_time,
                                                          self.input_queue.contents[0].name, resource_to_seize.name))
                self.input_queue.contents.pop(0)
                self.input_queue.contents.append(entity)
