"""
Created on: 23-8-2021 11:56

@author: IvS
"""
import pytest

from pydsol.pydsol_core.EventList import EventList


def test_remove_first(mocker):
    event_list = EventList()
    assert event_list.remove_first() is None

    # Event list has two events
    event_1 = mocker.Mock(absolute_execution_time=1)
    event_2 = mocker.Mock(absolute_execution_time=2)

    event_list.add(event_1)
    event_list.add(event_2)

    assert event_list.remove_first() == event_1
    assert event_1 not in event_list


def test_remove_last(mocker):
    event_list = EventList()
    assert event_list.remove_last() is None

    # Event list has two events
    event_1 = mocker.Mock(absolute_execution_time=1)
    event_2 = mocker.Mock(absolute_execution_time=2)

    event_list.add(event_1)
    event_list.add(event_2)

    assert event_list.remove_last() == event_2
    assert event_2 not in event_list


def test_first(mocker):
    # Event list is empty and is None
    event_list = EventList()
    assert event_list.first() is None

    # Event list has two events
    event_1 = mocker.Mock(absolute_execution_time=1)
    event_2 = mocker.Mock(absolute_execution_time=2)

    event_list.add(event_1)
    event_list.add(event_2)

    assert event_list.first() == event_1


def test_last(mocker):
    # Event list is empty and is None
    event_list = EventList()
    assert event_list.last() is None

    # Event list has two events
    event_1 = mocker.Mock(absolute_execution_time=1)
    event_2 = mocker.Mock(absolute_execution_time=2)

    event_list.add(event_1)
    event_list.add(event_2)

    assert event_list.last() == event_2


def test_cmp(mocker):
    mock_this = mocker.Mock()
    mock_other = mock_this

    event_list = EventList()

    # Test if objects are the same
    assert event_list._cmp(mock_this, mock_other) == 0

    # Test if not the same object and absolute execution time is different
    mock_other = mocker.Mock()

    mock_this.absolute_execution_time = 0
    mock_other.absolute_execution_time = 1
    assert event_list._cmp(mock_this, mock_other) == -1

    mock_this.absolute_execution_time = 2
    assert event_list._cmp(mock_this, mock_other) == 1

    # Test if same absolute execution time and priority is different
    mock_other.absolute_execution_time = 2

    mock_this.priority = 1
    mock_other.priority = 2
    assert event_list._cmp(mock_this, mock_other) == 1

    mock_this.priority = 3
    assert event_list._cmp(mock_this, mock_other) == -1

    # Test if same priority and id is different
    mock_other.priority = 3

    mock_this.id = 1
    mock_other.id = 2
    assert event_list._cmp(mock_this, mock_other) == -1

    mock_this.id = 3
    assert event_list._cmp(mock_this, mock_other) == 1

    # Check if ID is the same
    mock_other.id = 3
    with pytest.raises(AttributeError):
        event_list._cmp(mock_this, mock_other)


if __name__ == '__main__':
    pytest.main()
