"""
Created on: 21-9-2021 15:15

@author: IvS
"""
import pytest

from pydsol.pydsol_core.Simulator import Simulator
from pydsol.pydsol_core.SimEvent import SimEvent


def test_initialize(mocker):
    simulator = Simulator()

    model = mocker.Mock()
    replication = mocker.Mock()

    simulator.initialize(model, replication)


def test_warm_up(mocker):
    simulator = Simulator()
    mock_source = mocker.Mock()
    mock_target = mocker.Mock()
    mock_method = "method"

    simulator.warm_up(mock_source, mock_target, mock_method)

    mock_priority = 1

    simulator.warm_up(mock_source, mock_target, mock_method, mock_priority)


def test_schedule_event(mocker):
    simulator = Simulator()
    event = mocker.Mock()

    # Test Value Error
    event.absolute_execution_time = 1.0
    simulator.simulation_time = 1.1

    with pytest.raises(ValueError):
        simulator.schedule_event(event)

    # Test No Value Error
    event.absolute_execution_time = 1.2
    simulator.schedule_event(event)


def test_create_event_now(mocker):
    simulator = Simulator()
    mock_source = mocker.Mock()
    mock_target = mocker.Mock()
    mock_method = "method"

    event = simulator.create_simevent_now(mock_source, mock_target, mock_method)

    assert isinstance(event, SimEvent)


def test_start(mocker):
    simulator = Simulator()

    # Test Event list is Empty
    simulator.start()
    assert simulator.eventlist.is_empty() is True
    assert simulator.is_running is False

    # Test Event list has events
    simulator = Simulator()

    mock_source = mocker.Mock()
    mock_target = mocker.Mock()
    mock_method = "method"

    event = simulator.create_simevent(1, mock_source, mock_target, mock_method)
    simulator.schedule_event(event)

    # Test run time is smaller than absolute execution time
    simulator.run_time = 0.9
    simulator.start()
    assert simulator.is_running is False

    # Test run time is larger than absolute execution time
    simulator = Simulator()

    mock_source = mocker.Mock()
    mock_target = mocker.Mock()
    mock_method = "method"

    event = simulator.create_simevent(1, mock_source, mock_target, mock_method)
    simulator.schedule_event(event)

    simulator.run_time = 1.1
    simulator.start()
    assert simulator.simulation_time == event.absolute_execution_time == 1







