import numpy as np

from stylized_supply_chain_model.model_elements.product import Product
from pydsol.pydsol_core.Simulator import Simulator
from pydsol.pydsol_model.server import Server
from stylized_supply_chain_model.model_elements.statistic_manager import StatisticManager


class Manufacturer(Server, StatisticManager):
    def __init__(self, simulator: Simulator, processing_time, **kwargs):

        super().__init__(simulator, distribution=np.random.normal, processing_time=processing_time,
                         **kwargs)
        StatisticManager.__init__(self)

        self.location = kwargs['location']

    def seize_resource(self, entity: Product, **kwargs):
        self.quantity += entity.quantity
        super().seize_resource(entity, **kwargs)

    def enter_output_node(self, entity: Product, **kwargs):
        self.quantity -= entity.quantity
        super().enter_output_node(entity, **kwargs)

    def exit_output_node(self, entity, **kwargs):
        #Add interarrival delay
        interarrival_delay = entity.interarrival_distribution(*entity.interarrival_times)

        event = self.simulator.create_simevent(interarrival_delay, self, entity, "exit_with_vehicle")
        self.simulator.schedule_event(event)

    def exit_with_vehicle(self, entity, **kwargs):
        super().exit_output_node(entity, **kwargs)
