import numpy as np
from scipy.stats import skewnorm

from stylized_supply_chain_model.model_elements.product import Product
from pydsol.pydsol_model.server import Server
from stylized_supply_chain_model.model_elements.statistic_manager import StatisticManager

import logging
from pydsol.basic_logger import get_module_logger

logger = get_module_logger(__name__, level=logging.INFO)


class Port(Server, StatisticManager):
    def __init__(self, simulator, processing_time, **kwargs):
        super().__init__(simulator, distribution=skewnorm.rvs, processing_time=processing_time,
                         **kwargs)
        StatisticManager.__init__(self)
        self.location = kwargs['location']
        self.name = self.name + " " + self.location
        self.kwargs = kwargs

    def enter_input_node(self, entity, **kwargs):
        not_seized = [resource for resource in self.resources if not resource.resource_seized]
        new_transfer_in_time = np.random.triangular(*self.kwargs["transfer_in_time"]) if "transfer_in_time" in self.kwargs else 0

        for resource in not_seized:
            resource.transfer_in_time = new_transfer_in_time

        super().enter_input_node(entity, **kwargs)

    def seize_resource(self, entity: Product, **kwargs):
        self.quantity += entity.quantity
        super().seize_resource(entity, **kwargs)

    def enter_output_node(self, entity: Product, **kwargs):
        self.quantity -= entity.quantity

        if self.location == "Hai Phong":
            entity.start_international_transport = self.simulator.simulation_time

        super().enter_output_node(entity, **kwargs)

    def exit_output_node(self, entity, **kwargs):
        #Add interarrival delay
        interarrival_delay = entity.interarrival_distribution(*entity.interarrival_times)
        event = self.simulator.create_simevent(interarrival_delay, self, entity, "exit_with_vehicle")
        self.simulator.schedule_event(event)

    def exit_with_vehicle(self, entity, **kwargs):
        logger.debug(
            "Time {0:.2f}: {1} exits {2}".format(self.simulator.simulation_time, entity.name, self.name))
        super().exit_output_node(entity, **kwargs)
