import logging
import numpy as np

from stylized_supply_chain_model.model_elements.vehicles.small_truck import SmallTruck
from pydsol.pydsol_model.sink import Sink

from stylized_supply_chain_model.model_elements.statistic_manager import StatisticManager


class Retailer(Sink, StatisticManager):
    def __init__(self, simulator, transfer_in_time: [float, int], **kwargs):
        super().__init__(simulator, transfer_in_time=transfer_in_time
                          , **kwargs)
        StatisticManager.__init__(self)
        self.location = kwargs['location']
        self.name = self.name + " " + self.location

        self.entities_of_system = []

    def enter_input_node(self, entity: SmallTruck):
        self.quantity += np.sum([i.quantity for i in entity.entities_on_vehicle])
        super().enter_input_node(entity)

    def destroy_entity(self, entity: SmallTruck):
        for product in entity.entities_on_vehicle:
            product.time_in_system = self.simulator.simulation_time - product.start_time
            # Add value to list
            self.entities_of_system.append(product)
            self.quantity -= product.quantity
            super().destroy_entity(product)
        del entity

    def get_hourly_stats(self, obj):
        self.quantity = max(0, self.quantity)
        if self.quantity < 1:
            self.quantity = 0
        super().get_hourly_stats(obj)
