from stylized_supply_chain_model.model_elements.product import Product
from pydsol.pydsol_model.source import Source
from stylized_supply_chain_model.model_elements.statistic_manager import StatisticManager


class Supplier(Source, StatisticManager):
    def __init__(self, simulator, interarrival_time, **kwargs):
        super().__init__(simulator, interarrival_time, **kwargs)
        StatisticManager.__init__(self)
        self.location = kwargs['location']
        self.list_quantity = []

    def exit_source(self, entity: Product, **kwargs):
        # Tally quantity
        self.list_quantity.append(entity.quantity)
        self.quantity += entity.quantity
        super().exit_source(entity, **kwargs)

    def enter_output_node(self, entity: Product, **kwargs):
        self.quantity -= entity.quantity
        super().enter_output_node(entity, **kwargs)

    def exit_output_node(self, entity, **kwargs):
        #Add interarrival delay
        interarrival_delay = entity.interarrival_distribution(*entity.interarrival_times)

        event = self.simulator.create_simevent(interarrival_delay, self, entity, "exit_with_vehicle")
        self.simulator.schedule_event(event)

    def exit_with_vehicle(self, entity, **kwargs):
        super().exit_output_node(entity, **kwargs)
