"""
Created on: 13-8-2021 10:48

@author: IvS
"""
import copy

import numpy as np

from stylized_supply_chain_model.model_elements.product import Product
from pydsol.pydsol_core.Simulator import Simulator
from pydsol.pydsol_model.server import Server

from stylized_supply_chain_model.model_elements.statistic_manager import StatisticManager


class WholesalesDistributor(Server, StatisticManager):
    def __init__(self, simulator: Simulator,
                 minimum_processing_time: float,
                 mode_processing_time: float,
                 maximum_processing_time: float,
                 **kwargs):
        super().__init__(simulator, distribution=np.random.triangular,
                         processing_time=(minimum_processing_time, mode_processing_time, maximum_processing_time),
                         **kwargs)
        StatisticManager.__init__(self)
        # TODO: kwargs vervangen door een gewone parameter met default value?
        #  Zie https://www.geeksforgeeks.org/default-arguments-in-python/
        self.location = kwargs['location']
        self.kwargs = kwargs

        # Override function processing - Resource
        self.processing_function_resource = self.resources[0].processing
        for resource in self.resources:
            resource.processing = self.processing

    def seize_resource(self, entity: Product, **kwargs):
        entity.start_wholesales_distributor = self.simulator.simulation_time
        entity.international_transport_time = entity.start_wholesales_distributor - entity.start_international_transport

        self.quantity += entity.quantity

        super().seize_resource(entity, **kwargs)

    def processing(self, entity: Product, **kwargs):
        if "divide_quantity" in self.kwargs:
            batches_product = self.divide_entity(entity, self.kwargs["divide_quantity"])
            for batch in batches_product:
                self.processing_function_resource(batch, **kwargs)

    @staticmethod
    def divide_entity(entity: Product, copy_quantity: int):
        copies = []
        for i in range(copy_quantity):
            copy_entity = copy.copy(entity)
            copy_entity.quantity = entity.quantity / copy_quantity
            copy_entity.name = entity.name + "." + str(i)
            copies.append(copy_entity)
        return copies

    def get_hourly_stats(self, obj):
        self.quantity = max(0, self.quantity)
        if self.quantity < 1:
            self.quantity = 0
        super().get_hourly_stats(obj)

    def enter_output_node(self, entity: Product, **kwargs):
        self.quantity -= entity.quantity
        entity.wholesales_distributor_time = self.simulator.simulation_time - entity.start_wholesales_distributor

        super().enter_output_node(entity, **kwargs)

    def exit_output_node(self, entity, **kwargs):
        #Add interarrival delay
        interarrival_delay = entity.interarrival_distribution(*entity.interarrival_times)

        event = self.simulator.create_simevent(interarrival_delay, self, entity, "exit_with_vehicle")
        self.simulator.schedule_event(event)

    def exit_with_vehicle(self, entity, **kwargs):
        super().exit_output_node(entity, **kwargs)